import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;
import java.util.TreeSet;

public class DerivationTreeFinder {

	private final ContextFreeGrammar grammar;
	private final Map<String, Set<DerivationTree>> derivationTrees = new HashMap<>();

	public DerivationTreeFinder(ContextFreeGrammar grammar) {
		this.grammar = grammar;
		derivationTrees.put("", Collections.emptySet());
	}

	/**
	 * Computes the set of all derivation trees of the given word with respect to any non-terminal 
	 * of the grammar.
	 * TODO
	 */
	public Set<DerivationTree> derivationTrees(String w) {
		Set<DerivationTree> result = derivationTrees.get(w);
		if (result != null)
			return result;

		result = new HashSet<>();
		derivationTrees.put(w, result);

		if (w.length() == 1) {
			Terminal c = new Terminal(w.charAt(0));
			DerivationTree cTree = new DerivationTree(grammar, c);
			for (NonTerminal x : grammar.getProductionsByRhs(Collections.singletonList(c)))
				result.add(new DerivationTree(grammar, x, cTree));
		} else {
			for (int k = 1; k < w.length(); k++) {
				String w1 = w.substring(0, k), w2 = w.substring(k);
				for (DerivationTree left : derivationTrees(w1))
					for (DerivationTree right : derivationTrees(w2))
						for (NonTerminal lhs : grammar.getProductionsByRhs(List.of(left.symbol, right.symbol)))
							result.add(new DerivationTree(grammar, lhs, left, right));
			}
		}

		return result;
	}

	public static void main(String[] args) {
		Scanner sc = new Scanner(System.in);
		ContextFreeGrammar g = ContextFreeGrammar.parse(sc);
		DerivationTreeFinder finder = new DerivationTreeFinder(g);
		String w = sc.nextLine();
		TreeSet<DerivationTree> ts = new TreeSet<>(finder.derivationTrees(w));
		for (DerivationTree t : ts)
			System.out.println(t);
		sc.close();
	}

}
