import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Scanner;
import java.util.Set;
import java.util.TreeSet;

public class Productivity {

	private ContextFreeGrammar grammar;
	private Set<NonTerminal> productive;

	public Productivity(ContextFreeGrammar grammar) {
		this.grammar = grammar;
		build();
	}

	/**
	 * Fill the set "productive" with the set of all productive symbols.
	 */
	private void build() {
		productive = new HashSet<>();
		boolean changed; // Has anything changed in the last iteration?
		do {
			changed = false;
			for (Production p : grammar.productions) {
				if (productive.contains(p.left))
					continue;
				if (p.right.stream().allMatch(x -> x instanceof Terminal || productive.contains((NonTerminal) x))) {
					productive.add(p.left);
					changed = true;
				}
			}
		} while (changed);
	}

	private Set<NonTerminal> get() {
		return Collections.unmodifiableSet(productive);
	}

	public static Set<NonTerminal> getProductiveNonTerminals(ContextFreeGrammar g) {
		return (new Productivity(g)).get();
	}

	public static void main(String[] args) {
		Scanner sc = new Scanner(System.in);
		ContextFreeGrammar g = ContextFreeGrammar.parse(sc);
		for (NonTerminal x : new TreeSet<>(getProductiveNonTerminals(g)))
			System.out.println(x);
		sc.close();
	}

}
