import java.util.ArrayDeque;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Queue;
import java.util.Scanner;
import java.util.Set;
import java.util.TreeSet;

public class Reachability {

	private ContextFreeGrammar grammar;
	private Set<NonTerminal> reachable;

	public Reachability(ContextFreeGrammar grammar) {
		this.grammar = grammar;
		build();
	}

	/**
	 * Fill the set "reachable" with the set of all productive symbols.
	 */
	private void build() {
		reachable = new HashSet<>();
		Queue<NonTerminal> workList = new ArrayDeque<>();
		reachable.add(grammar.startSymbol);
		workList.offer(grammar.startSymbol);

		while (!workList.isEmpty()) {
			NonTerminal c = workList.poll();
			for (List<Symbol> rhs : grammar.getProductionsByLhs(c)) {
				for (Symbol x : rhs) {
					if (x instanceof NonTerminal && !reachable.contains((NonTerminal) x)) {
						reachable.add((NonTerminal) x);
						workList.offer((NonTerminal) x);
					}
				}
			}
		}
	}

	private Set<NonTerminal> get() {
		return Collections.unmodifiableSet(reachable);
	}

	public static Set<NonTerminal> getReachableNonTerminals(ContextFreeGrammar g) {
		return (new Reachability(g)).get();
	}

	public static void main(String[] args) {
		Scanner sc = new Scanner(System.in);
		ContextFreeGrammar g = ContextFreeGrammar.parse(sc);
		for (NonTerminal x : new TreeSet<>(getReachableNonTerminals(g)))
			System.out.println(x);
		sc.close();
	}

}
